// An ADD comment will be used to indicate variables and code added
// to the shader to increase functionality.
// The CHANGE comment will show changes made to demo shader code from 
// the previous version.

#version 150 core

in vec4 Color; 

// ADD
// We add a float (to act as a boolean) to turn lights ON/OFF.
in float LightsOn;

// ADD
// Position stores the point of the fragment in eyeCoordinates for computation 
// of relative light directions.
in vec3 Position;

// ADD
// Normal is the transformed normal vector for the fragment.
in vec3 Normal;

// ADD
// Receive back-transformed lightPosition so that light remains fixed under
// object rotation.
in vec4 lightPosition;

out vec4 fragmentColor;

void main()
{
    // CHANGE
    // We add this conditional (0.0f = false) to carry out lighting computations
    // or let the color pass through.
    if (LightsOn>0.5) {
        // Computations modeled after Chapter 2, 3 of...
        // OpenGL 4.0 Shading Language Cookbook by David Wolff (2011).

        // Normalize the transformed normal vector - general code, unnecessary 
        // in this case as the vector should already be normalized.
	vec3 n = normalize(Normal);

        // Where is the light relative to the fragment?
	vec3 s = normalize(vec3(lightPosition) - Position);

        // Where is the viewer relative to the fragment?
	vec3 v = normalize(-Position);

        // Direction of reflected light
	vec3 r = reflect(-s, n);

        // Half-vector to reduce numerical operations since processing many 
        // fragments.
	vec3 h = normalize(v + s);

        // Hard code - ambient surface reflectivity and light color/intensity.
	vec3 Ka = vec3(0.3, 0.3, 0.3);
	vec3 La = vec3(Color);

        // Hard code - diffuse surface reflectivity and light color/intensity.
	vec3 Kd = vec3(0.75, 0.75, 0.75);
	vec3 Ld = vec3(Color);
	
        // Hard code - specular surface reflectivity and light color/intensity.
	vec3 Ks = vec3(0.2, 0.2, 0.2);
	vec3 Ls = vec3(1.0,1.0,1.0);
	
        // Compute ambient and diffuse color and intensity.
	vec3 ambient = La * Ka;
	float sDotN = max( dot(s, n), 0.0);
	vec3 diffuse = Ld * Kd * sDotN;
	
        // Compute specular component.
	vec3 spec = vec3(0.0);	
	float Shininess = 25;
	if (sDotN > 0.0)
	{
		spec = Ls * Ks * pow(max(dot(h,n), 0.0), Shininess);
	}

        // Compute overall fragment illumination.
	vec3 res = ambient + diffuse + spec;
	
        // And combine with alpha value for final fragment color.
	fragmentColor = vec4(res, Color.a); 
    } else {
        // Otherwise, we simply pass on the incoming rgb color and alpha.
        // Color format is (red, green, blue, alpha [aka transparency: 1=opaque]).
        // Color component and alpha range is a float, 0.0f-1.0f
        fragmentColor = Color; 
    }
}
